/* shared.c
 * Shared forest nodes.
 */

#include "shared.h"

dot is_dot_format = FALSE;

/** Generates an empty shared node. */
static inline shared
empty_rule (char *name)
{
  shared ret;

  ret.nam = name;
  ret.pos = yytokenpos;
  ret.len = 0;

  return ret;
}

/** Prints a single shared node name. */
inline void
print_shared (FILE *fp, shared s)
{
  fprintf(fp, "%s_%d_%d", s.nam, s.pos, s.len);
}


/** Generates a shared token. */
shared
token (char* name)
{
  shared ret = empty_rule (name);
  ret.len = 1;
  return ret;
}

/** Generates a shared node with its additional arguments as sons. */
shared
shared_rule (char* name, unsigned short n, ...)
{
  shared ret;

  ret = empty_rule(name);

  if (n)
    {
      shared cur;
      unsigned short len, i;
      va_list ap;

      va_start(ap, n);
      cur     = va_arg(ap, shared);
      ret.pos = cur.pos;
      ret.len = cur.len;
 
      for (i = 1; i < n; i++)
        ret.len += va_arg(ap, shared).len;

      va_end(ap);
    }
  return ret;
}

/** Prints a shared node production. */
void
print_rule (shared s, unsigned short n, ...)
{
  if (!is_dot_format)
    {
      print_shared(stdout, s);
      printf(" -> ");

      if (n)
        {
          unsigned short i;
          va_list ap;
          
          va_start(ap, n);
          
          for (i = 0; i < n; i++)
            {
              print_shared(stdout, va_arg(ap, shared));
              printf(" ");
            }
          
          va_end(ap);
        }
      printf("\n");
    }
  else if (n)
    {
      unsigned short i;
      va_list ap;
          
      va_start(ap, n);
          
      for (i = 0; i < n; i++)
        {
          print_shared(stdout, s);
          printf(" -> ");
          print_shared(stdout, va_arg(ap, shared));
          printf(";\n");          
        }
    }
}  
